package org.netkernel.client.http.endpoint;

import org.netkernel.client.http.representation.HttpClientConfig;
import org.netkernel.client.http.representation.IHttpClientConfig;
import org.netkernel.container.config.IConfiguration;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;

public class HttpConfigTransreptor extends StandardTransreptorImpl
{	
	public HttpConfigTransreptor()
	{	this.declareThreadSafe();
		this.declareToRepresentation(IHttpClientConfig.class);
	}

	public void onTransrept(INKFRequestContext aContext) throws Exception
	{	
		boolean followRedirects=true;
		long acceptableContentLength=-1;
		int connectionsPerHost=4;
		int totalConnections=10;
		int stateExpirationTime=600;
		int retryAttempts=3;
		int connectTimeout=2000;
		int timeout=5000;
		int proxyPort=80;
		boolean expectContinue=true;
		String userAgent="NetKernel 4.x.x. active:httpXXXX";
		
		IConfiguration kernelConfig=aContext.getKernelContext().getKernel().getConfiguration();
		IHDSNode cfg=aContext.sourcePrimary(IHDSNode.class).getFirstNode("/*");

		//proxy values default to values from kernel configuration
		String proxyHost=getProxyValue("proxyHost", kernelConfig, cfg);
		String sproxyPort=getProxyValue("proxyPort", kernelConfig, cfg);
		if(sproxyPort!=null)
		{	try
			{	proxyPort=Integer.parseInt(sproxyPort);
			}
			catch(Exception e)
			{	e.printStackTrace();
			}
		}
		String proxyUser=getProxyValue("proxyUsername", kernelConfig, cfg);
		String proxyPassword=getProxyValue("proxyPassword", kernelConfig, cfg);
		String proxyNTLMHost=getProxyValue("proxyNTWorkstation", kernelConfig, cfg);
		String proxyNTLMDomain=getProxyValue("proxyDomain", kernelConfig, cfg);		
		
		cfg=cfg.getFirstNode("/config");
		if(cfg.getFirstNode("followRedirects")!=null)
		{	String temp=cfg.getFirstValue("followRedirects").toString();
			followRedirects=temp.startsWith("t") || temp.equals("1");
		}
		if(cfg.getFirstNode("retryAttempts")!=null)
		{	String temp=cfg.getFirstValue("retryAttempts").toString();
			retryAttempts=Integer.parseInt(temp);
		}
		if(cfg.getFirstNode("maxConnectionsPerHost")!=null)
		{	String temp=cfg.getFirstValue("maxConnectionsPerHost").toString() ;
			connectionsPerHost=Integer.parseInt(temp);
		}
		if(cfg.getFirstNode("maxTotalConnections")!=null)
		{	String temp=cfg.getFirstValue("maxTotalConnections").toString();
			totalConnections=Integer.parseInt(temp);
		}
		if(cfg.getFirstNode("maxAcceptableContentLength")!=null)
		{	String temp=cfg.getFirstValue("maxAcceptableContentLength").toString();
			acceptableContentLength=Long.parseLong(temp);
		}
		if(cfg.getFirstNode("stateExpirationTime")!=null)
		{	String temp=cfg.getFirstValue("stateExpirationTime").toString();
			stateExpirationTime=Integer.parseInt(temp);
		}
		if(cfg.getFirstNode("connectTimeout")!=null)
		{	String temp=cfg.getFirstValue("connectTimeout").toString();
			connectTimeout=Integer.parseInt(temp);
		}
		if(cfg.getFirstNode("timeout")!=null)
		{	String temp=cfg.getFirstValue("timeout").toString();
			timeout=Integer.parseInt(temp);
		}
		if(cfg.getFirstNode("expectContinue")!=null)
		{	String temp=cfg.getFirstValue("expectContinue").toString();
			expectContinue=temp.startsWith("t") || temp.equals("1");
		}
		if(cfg.getFirstNode("userAgent")!=null)
		{	String temp=cfg.getFirstValue("userAgent").toString();
			userAgent=temp;
		}
		
		HttpClientConfig ca=new HttpClientConfig(
			followRedirects,
			acceptableContentLength,
			connectionsPerHost,
			totalConnections,
			stateExpirationTime,
			retryAttempts,
			connectTimeout,
			timeout,
			proxyHost, 
			proxyPort,
			proxyUser,
			proxyPassword,
			proxyNTLMHost,
			proxyNTLMDomain,
			expectContinue,
			userAgent
			);
		
		aContext.createResponseFrom(ca);
	}
	
	private static String getProxyValue(String aName, IConfiguration aKernelConfig, IHDSNode aTransreptConfig)
	{	String result=(String)aTransreptConfig.getFirstValue(aName);
		if (result==null)
		{	result=aKernelConfig.getString("http."+aName,null);
			if (result!=null && (result.equals("") || result.startsWith("%") )) result=null; //it might be an uninitialised substitution value
		}
		return result;
	}
	

}
